import openimagelib
import openmedialib
import urllib
import socket
import os
import select

# A reader which handles http stream playback
#
# NB: It assumes that the url to open starts with oml:http or oml:ftp

class url_reader( openmedialib.stream_handler_delegate ):
	def __init__( self ):
		openmedialib.stream_handler_delegate.__init__( self )
		self.stream = None
		self.block = ""

	def open( self, url, flags ):
		self.stream = urllib.urlopen( url[ 4: ] )
		return self.stream is not None

	def read( self, size ):
		# Poll the connection first
		ready = select.select( [ self.stream ], [ ], [ ], 0.0 )[ 0 ]
		# If nothing shows immediately, try for a slightly longer period
		if len( ready ) == 0:
			ready = select.select( [ self.stream ], [ ], [ ], 5.0 )[ 0 ]
			if len( ready ) == 0:
				self.block = ""
				return self.block
		self.block = self.stream.read( size )
		return self.block

	def write( self, data ):
		return -1

	def seek( self, pos, whence ):
		return -1

	def close( self ):
		if self.stream is not None:
			self.stream.close( )
			self.stream = None
			return 0
		return -1

	def is_stream( self ):
		return True

	def is_thread_safe( self ):
		return False

# A reader which handles http stream playback and capture
#
# NB: It assumes that the url to open starts with oml:dump:file@url

class url_dump( openmedialib.stream_handler_delegate ):
	def __init__( self ):
		openmedialib.stream_handler_delegate.__init__( self )
		self.stream = None
		self.block = ""

	def open( self, url, flags ):
		try:
			self.local, self.remote = url[ len( "oml:dump:" ): ].split( "@", 1 )
			if self.local == "" or os.path.isdir( self.local ):
				self.local = os.path.join( self.local, os.path.split( self.remote )[ 1 ] )
			self.dump = open( self.local, "wb" )
			self.stream = urllib.urlopen( self.remote )
			return self.dump is not None and self.stream is not None
		except Exception, e:
			print "Malformed url " + url
			return False

	def read( self, size ):
		self.block = self.stream.read( size )
		self.dump.write( self.block )
		return self.block

	def write( self, data ):
		return -1

	def seek( self, pos, whence ):
		return -1

	def close( self ):
		if self.dump is not None:
			self.dump.close( )
		if self.stream is not None:
			self.stream.close( )
		return 0

	def is_stream( self ):
		return True

	def is_thread_safe( self ):
		return False

# A reader/writer test case
#
# NB: It assumes that the url to open starts with oml:

class file_handler( openmedialib.stream_handler_delegate ):
	def __init__( self ):
		openmedialib.stream_handler_delegate.__init__( self )
		self.stream = None
		self.block = ""

	def open( self, url, flags ):
		mode = "rb"
		if flags == 1:
			mode = "wb+"
		self.stream = open( url[ 4: ], mode )
		return self.stream is not None

	def read( self, size ):
		try:
			self.block = self.stream.read( size )
		except Exception:
			self.block = ""
		return self.block

	def write( self, data ):
		try:
			result = self.stream.write( data )
		except Exception:
			data = ""
		return len( data )

	def seek( self, pos, whence ):
		if whence == 65536: whence = 2
		result = self.stream.seek( pos, whence )
		return self.stream.tell( )

	def close( self ):
		if self.stream is not None:
			self.stream.close( )
			self.stream = None
			return 0
		return -1

	def is_stream( self ):
		return False

	def is_thread_safe( self ):
		return False

# TCP Reader/Writer
#
# It assumes that the url to open starts with oml:tcp[:server[:port]]:.ext
# where server defaults to '' or localhost (depending on write or read)
#       port defaults to 51630
#       extension must be mpg, dv, ts or whatever is being generated
#
# It shouldn't matter whether reader or writer are opened first - the first
# one in will bind the port and wait indefinitely for a connection (hence, it 
# behaves precisely like a fifo).
#
# Caveat: Does this introduce a race condition? Is there a cleaner solution?

class tcp_handler( openmedialib.stream_handler_delegate ):
	def __init__( self ):
		openmedialib.stream_handler_delegate.__init__( self )
		self.stream = None
		self.block = ""

	def open( self, url, flags ):
		self.stream = None
		self.socket = None

		parse_error = False
		server = ''
		port = 51630
		server_part = url[ 7: ]

		try:
			if server_part.startswith( ":" ):
				split = server_part[ 1: ].split( ":" )
				if len( split ) == 3:
					server = split[ 0 ]
					port = int( split[ 1 ] )
				elif len( split ) == 2:
					port = int( split[ 0 ] )
				elif len( split ) == 1:
					pass
				else:
					parse_error = True
		except Exception:
			parse_error = True

		if not parse_error:
			if flags == 0 and server == '':
				server = "localhost"
			self.socket = socket.socket( socket.AF_INET, socket.SOCK_STREAM )
			self.socket.setsockopt( socket.SOL_SOCKET, socket.SO_REUSEADDR, 1 )
			try:
				self.socket.bind( ( server, port ) )
				self.socket.listen( 5 )
				self.stream, address = self.socket.accept( )
			except socket.error, e:
				self.stream = socket.socket( socket.AF_INET, socket.SOCK_STREAM )
				try:
					self.stream.connect( ( server, port ) )
				except socket.error:
					self.stream = None

		return self.stream is not None

	def read( self, size ):
		try:
			self.block = self.stream.recv( size )
		except Exception:
			self.block = ""
		return self.block

	def write( self, data ):
		try:
			result = self.stream.send( data )
		except Exception:
			result = -1
		return result

	def seek( self, pos, whence ):
		return -1

	def close( self ):
		if self.stream is not None:
			self.stream.close( )
			self.stream = None
			if self.socket is not None:
				self.socket.close( )
			return 0
		return -1

	def is_stream( self ):
		return True

	def is_thread_safe( self ):
		return False

# Factory method for handlers

def url_callback( url, flags ):
	if flags == 0 and ( url.startswith( "oml:http:" ) or url.startswith( "oml:ftp:" ) ):
		return url_reader( )
	elif flags == 0 and url.startswith( "oml:dump:" ):
		return url_dump( )
	elif url.startswith( "oml:tcp:" ):
		return tcp_handler( )
	elif url.startswith( "oml:" ) and flags in ( 0, 1 ):
		return file_handler( )
	else:
		return None

